﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ws.theia.minecraft.bukkit.permissionsEx.editor {
	public partial class editTab:UserControl {
		public editTab() {
			InitializeComponent();
		}

		#region 動作モード

		/// <summary>
		/// 動作モード
		/// </summary>
		private int _mode;

		/// <summary>
		/// 動作モード
		/// </summary>
		[Category("動作")]
		[DefaultValue(0)]
		[Description("コントロールの動作モードを設定します。")]
		public virtual int mode {
			set {
				this._mode=value;
				if(this.mode==0) {
					this.userListLabel.Text="グループ一覧(&G)";
				} else {
					this.userListLabel.Text="ユーザ一覧(&U)";
				}
			}
			private get {
				return this._mode;
			}
		}

		#endregion

		#region コネクタ

		[Category("動作")]
		[DefaultValue(null)]
		[Description("コネクタを設定します。")]
		public virtual mysqlConnector connector {
			private get;
			set;
		}

		#endregion

		#region ローカルフィールド

		/// <summary>
		/// 使用データテーブルコレクション
		/// </summary>
		private Dictionary<string,mysqlConnector.dataTableAndAdapter> dataTables=new Dictionary<string,mysqlConnector.dataTableAndAdapter>();

		#endregion

		#region イベントハンドラ

		/// <summary>
		/// セーブボタンクリック処理
		/// </summary>
		/// <param name="sender">イベント送信オブジェクト</param>
		/// <param name="e">イベント引数</param>
		private void saveBtn_Click(object sender,EventArgs e) {
			this.commit();
		}

		/// <summary>
		/// ロールバックボタンクリック処理
		/// </summary>
		/// <param name="sender">イベント送信オブジェクト</param>
		/// <param name="e">イベント引数</param>
		private void rollbackBtn_Click(object sender,EventArgs e) {
			this.rollback();
		}

		/// <summary>
		/// 権限グリッドビューカレントセル値変更イベント
		/// </summary>
		/// <param name="sender">イベント送信オブジェクト</param>
		/// <param name="e">イベント引数</param>
		private void permissinDataGridView_CellValueChanged(object sender,DataGridViewCellEventArgs e) {

			//保存、ロールバックボタン有効状態切り替え
			this.changeSaveAndRollbackBtnEnable();

		}

		/// <summary>
		/// 権限グリッドビューカレントセル変更イベント
		/// </summary>
		/// <param name="sender">イベント送信オブジェクト</param>
		/// <param name="e">イベント引数</param>
		private void permissinDataGridView_CurrentCellChanged(object sender,EventArgs e) {

			//保存、ロールバックボタン有効状態切り替え
			this.changeSaveAndRollbackBtnEnable();

		}

		/// <summary>
		/// エンティティリストグリッドビューカレントセル値変更イベント
		/// </summary>
		/// <param name="sender">イベント送信オブジェクト</param>
		/// <param name="e">イベント引数</param>
		private void UserListDataGridView_CellValueChanged(object sender,DataGridViewCellEventArgs e) {

			//保存、ロールバックボタン有効状態切り替え
			this.changeSaveAndRollbackBtnEnable();

		}

		/// <summary>
		/// エンティティリストグリッドビューカレントセル変更イベント
		/// </summary>
		/// <param name="sender">イベント送信オブジェクト</param>
		/// <param name="e">イベント引数</param>
		private void UserListDataGridView_CurrentCellChanged(object sender,EventArgs e) {
			if(this.UserListDataGridView.CurrentRow!=null) {

				//権限リストを権限リストグリッドビューへ登録
				this.setDataTable(this.permissinDataGridView,connector.getPermissionsTN(this.mode,this.UserListDataGridView.CurrentRow.Cells["name"].Value.ToString()),new getTableMethod(this.getPermissions),this.UserListDataGridView.CurrentRow.Cells["name"].Value.ToString());

				//権限リストを権限リストグリッドビューのヘッダ設定
				this.headerHide(this.permissinDataGridView,"permission","world","value");
				this.permissinDataGridView.ColumnHeadersVisible=true;
				this.permissinDataGridView.Columns["permission"].HeaderText="権限";
				this.permissinDataGridView.Columns["permission"].Width=(int)((this.permissinDataGridView.Width-150-20)*0.6);
				this.permissinDataGridView.Columns["world"].HeaderText="対象ワールド名";
				this.permissinDataGridView.Columns["world"].Width=150;
				this.permissinDataGridView.Columns["value"].HeaderText="設定値";
				this.permissinDataGridView.Columns["value"].Width=(int)((this.permissinDataGridView.Width-150-20)*0.4);

				//継承リストを継承リストグリッドビューへ登録
				this.setDataTable(this.inhelitanceDataGridView,connector.getInheritanceTN(this.mode,connector.getInheritanceTN(this.mode,this.UserListDataGridView.CurrentRow.Cells["name"].Value.ToString())),new getTableMethod(this.getInheritance),this.UserListDataGridView.CurrentRow.Cells["name"].Value.ToString());

				//継承リストグリッドビューのヘッダ設定
				this.headerHide(this.inhelitanceDataGridView,"parent","world");
				this.inhelitanceDataGridView.ColumnHeadersVisible=true;
				this.inhelitanceDataGridView.Columns.Clear();
				var parentCol=new DataGridViewComboBoxColumn() {
					DataPropertyName="parent",
					Name="parent",
					ValueMember="name",
					DisplayMember="name"
				};
				parentCol.DataSource=this.connector.getEntityList(0,this.UserListDataGridView.CurrentRow.Cells["name"].Value.ToString());
				parentCol.Width=150;
				parentCol.HeaderText="権限継承元グループ名";
				this.inhelitanceDataGridView.Columns.Add(parentCol);

				var worldCol=new DataGridViewTextBoxColumn() {
					DataPropertyName="world",
					Name="world",
					HeaderText="対象ワールド名",
					Width=150
				};
				this.inhelitanceDataGridView.Columns.Add(worldCol);

				//保存、ロールバックボタン有効状態切り替え
				this.changeSaveAndRollbackBtnEnable();

			}
		}

		/// <summary>
		/// 継承グリッドビューカレントセル値変更イベント
		/// </summary>
		/// <param name="sender">イベント送信オブジェクト</param>
		/// <param name="e">イベント引数</param>
		private void inhelitanceDataGridView_CellValueChanged(object sender,DataGridViewCellEventArgs e) {

			//保存、ロールバックボタン有効状態切り替え
			this.changeSaveAndRollbackBtnEnable();

		}

		/// <summary>
		/// 継承グリッドビューカレントセル変更イベント
		/// </summary>
		/// <param name="sender">イベント送信オブジェクト</param>
		/// <param name="e">イベント引数</param>
		private void inhelitanceDataGridView_CurrentCellChanged(object sender,EventArgs e) {

			//保存、ロールバックボタン有効状態切り替え
			this.changeSaveAndRollbackBtnEnable();

		}

		#endregion

		#region トランザクションコントロール

		/// <summary>
		/// エンティティリストをロード
		/// </summary>
		public void loadEntityList() {

			//エンティティリストグリッドビューのカレント行を所得
			var currentRow=0;
			if(this.UserListDataGridView.CurrentRow!=null) {
				currentRow=this.UserListDataGridView.CurrentRow.Index;
			}

			//エンティティリストをエンティティリストグリッドビューに登録
			this.setDataTable(this.UserListDataGridView,connector.getEntityListTN(this.mode),new getTableMethod(this.getEntityList),"");
			this.headerHide(this.UserListDataGridView,"name");

			//直前のカレント行反映処理
			if(this.UserListDataGridView.Rows.Count>=currentRow) {
				if(this.UserListDataGridView.Columns.Contains("name")) {
					this.UserListDataGridView.CurrentCell=this.UserListDataGridView["name",currentRow];
				}
			}

		}


		#region データテーブル取得

		/// <summary>
		/// 権限テーブル取得
		/// </summary>
		/// <param name="name">取得対象名</param>
		/// <returns></returns>
		public mysqlConnector.dataTableAndAdapter getPermissions(string name) {
			return connector.getPermissions(this.mode,name);
		}

		/// <summary>
		/// エンティティテーブル取得
		/// </summary>
		/// <param name="name">取得対象名</param>
		public mysqlConnector.dataTableAndAdapter getEntityList(string name) {
			return connector.getEntityList(this.mode);
		}

		/// <summary>
		/// 継承テーブル取得
		/// </summary>
		/// <param name="name">取得対象名</param>
		public mysqlConnector.dataTableAndAdapter getInheritance(string name) {
			return connector.getInheritance(this.mode,name);
		}

		/// <summary>
		///　テーブル取得メソッドデリゲート
		/// </summary>
		/// <param name="name">取得対象名</param>
		/// <returns></returns>
		private delegate mysqlConnector.dataTableAndAdapter getTableMethod(string name);

		#endregion

		#region データテーブル設定処理

		/// <summary>
		/// データテーブル設定処理
		/// </summary>
		/// <param name="dgv">設定対象データグリッドビュー</param>
		/// <param name="tableName">テーブル名</param>
		/// <param name="gTM">テーブル取得メソッド</param>
		/// <param name="name">取得対象名</param>
		private void setDataTable(DataGridView dgv,string tableName,getTableMethod gTM,string name) {
			if(this.dataTables.ContainsKey(tableName)) {
				dgv.DataSource=this.dataTables[tableName].dataTable;
			} else {
				var dtaa=gTM(name);
				dgv.DataSource=dtaa.dataTable;
				if(this.dataTables.ContainsKey(tableName)) {
					this.dataTables.Add(dtaa.dataTable.TableName,dtaa);
				}
			}
		}

		#endregion


		/// <summary>
		/// 変更をコミット
		/// </summary>
		public void commit() {

			//テーブルの変更内容を反映
			foreach(KeyValuePair<string,mysqlConnector.dataTableAndAdapter> keyVal in this.dataTables) {
				keyVal.Value.commit();
			}

			//トランザクションをコミット
			this.connector.commit();

			//使用データテーブルリストをクリア
			this.dataTables.Clear();

			//エンティティリストをリロード
			this.loadEntityList();

			//保存ボタン、ロールバックボタンを非活性化
			this.changeSaveAndRollbackBtnEnable();

		}

		/// <summary>
		/// 変更をロールバック
		/// </summary>
		public void rollback() {

			//トランザクションをロールバック
			this.connector.rollback();

			//使用データテーブルリストをクリア
			this.dataTables.Clear();

			//エンティティリストをリロード
			this.loadEntityList();

			//保存ボタン、ロールバックボタンを非活性化
			this.changeSaveAndRollbackBtnEnable();

		}

		/// <summary>
		/// 未保存テーブル数取得
		/// </summary>
		public int unSaveTable {
			get {

				//未保存テーブルカウンタ
				var changeTable=0;

				//各テーブルの変更状況を取得
				foreach(KeyValuePair<string,mysqlConnector.dataTableAndAdapter> dtObj in this.dataTables) {

					//変更データがNULLでなければ未保存テーブルとしてカウント
					if(dtObj.Value.dataTable.GetChanges()!=null) {
						changeTable++;
					}

				}
				return changeTable;
			}
		}

		#endregion

		#region コネクションコントロール

		/// <summary>
		/// コネクションクローズ処理
		/// </summary>
		public void closeConnection() {

			//ロールバック
			this.rollback();

			//各グリッドビューをクリア
			this.UserListDataGridView.DataSource=null;
			this.permissinDataGridView.DataSource=null;
			this.inhelitanceDataGridView.DataSource=null;

			//コネクタをクリア
			this.connector=null;

		}

		#endregion

		#region フォームコントロール表示変更

		/// <summary>
		/// 保存、ロールバックボタン有効状態切り替え処理
		/// </summary>
		private void changeSaveAndRollbackBtnEnable() {
			var changeTableCount=this.unSaveTable;
			this.saveBtn.Enabled=changeTableCount >0;
			this.rollbackBtn.Enabled=changeTableCount>0;
		}

		/// <summary>
		/// 不要ヘッダ非表示処理
		/// </summary>
		/// <param name="dgv">対象データグリッドビュー</param>
		/// <param name="visibleCols">表示カラムリスト</param>
		private void headerHide(DataGridView dgv,params string[] visibleCols) {

			//ヘッダ類を非表示にする
			dgv.ColumnHeadersVisible=false;
			dgv.RowHeadersVisible=false;

			//不要列を非表示にする
			foreach(DataGridViewColumn dgvc in dgv.Columns) {
				dgvc.Visible=Array.IndexOf(visibleCols,dgvc.Name)>=0;
			}

		}

		#endregion

	}
}
