﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ws.theia.minecraft.bukkit.permissionsEx.editor {
	public class mysqlConnector {

		#region コンストラクタ


		/// <summary>
		/// コンストラクタ
		/// </summary>
		/// <param name="Server">サーバアドレス</param>
		/// <param name="Database">データベースネーム</param>
		/// <param name="UserID">ユーザID</param>
		/// <param name="Password">パスワード</param>
		public mysqlConnector(string Server,string Database,string UserID,string Password)
			: this(new MySql.Data.MySqlClient.MySqlConnectionStringBuilder() {
				Server=Server,
				Database=Database,
				UserID=UserID,
				Password=Password
			}) {
		}

		/// <summary>
		/// コンストラクタ
		/// </summary>
		/// <param name="cStrBuilder">コネクションストリングビルダ</param>
		public mysqlConnector(MySql.Data.MySqlClient.MySqlConnectionStringBuilder cStrBuilder)
			: this(new MySql.Data.MySqlClient.MySqlConnection(cStrBuilder.ConnectionString)) {

		}

		/// <summary>
		/// コンストラクタ
		/// </summary>
		/// <param name="connection">コネクション</param>
		public mysqlConnector(MySql.Data.MySqlClient.MySqlConnection connection) {

			//コネクションをセット
			this.connection=connection;

			//コネクションをオープン
			this.open();

		}

		#endregion

		#region ローカルフィールド

		/// <summary>
		/// MySQLコネクタ
		/// </summary>
		private MySql.Data.MySqlClient.MySqlConnection connection;

		/// <summary>
		/// トランザクションオブジェクト
		/// </summary>
		private MySql.Data.MySqlClient.MySqlTransaction transaction;

		#endregion

		#region テーブル名取得

		/// <summary>
		/// パーミッションリストテーブル名取得
		/// </summary>
		/// <param name="mode">取得モード</param>
		/// <param name="name">取得対象名称</param>
		/// <returns></returns>
		public string getPermissionsTN(int mode,string name) {
			return "permissions:"+mode.ToString()+":"+name;
		}

		/// <summary>
		/// エンティティリストテーブル名取得取得
		/// </summary>
		/// <param name="mode">取得モード</param>
		/// <returns></returns>
		public string getEntityListTN(int mode) {
			return "permissions_entity:"+mode.ToString();
		}

		/// <summary>
		/// 継承リスト取得
		/// </summary>
		/// <param name="mode">取得モード</param>
		/// <param name="name">取得対象名称</param>
		/// <returns></returns>
		public string getInheritanceTN(int mode,string name) {
			return "permissions_inheritance:"+mode.ToString()+":"+name;
		}

		#endregion

		#region リスト取得

		/// <summary>
		/// パーミッションリスト取得
		/// </summary>
		/// <param name="mode">取得モード</param>
		/// <param name="name">取得対象名称</param>
		/// <returns></returns>
		public dataTableAndAdapter getPermissions(int mode,string name) {

			//データテーブル取得
			var dtaa=this.getData("SELECT * FROM permissions WHERE type="+mode.ToString()+" AND name='"+name+"' AND permission<>'rank'",this.getPermissionsTN(mode,name),mode,name);

			//コミットイベント登録
			dtaa.commiting +=this.forPermissions;

			return dtaa;

		}

		/// <summary>
		/// エンティティリスト取得
		/// </summary>
		/// <param name="mode">取得モード</param>
		/// <returns></returns>
		public dataTableAndAdapter getEntityList(int mode) {

			//データテーブル取得
			var dtaa=this.getData("SELECT * FROM permissions_entity WHERE type="+mode.ToString(),this.getEntityListTN(mode),mode,"");

			//コミットイベント登録
			dtaa.commiting+=this.forEntityList;
				
			return dtaa;

		}

		public System.Data.DataTable getEntityList(int mode,string name) {
			//データテーブル取得
			var dtaa=this.getData("SELECT * FROM permissions_entity WHERE type="+mode.ToString(),this.getEntityListTN(mode)+" AND name<>'" +name+ "'",mode,"");

			return dtaa.dataTable;
		}

		/// <summary>
		/// 継承リスト取得
		/// </summary>
		/// <param name="mode">取得モード</param>
		/// <param name="name">取得対象名称</param>
		/// <returns></returns>
		public dataTableAndAdapter getInheritance(int mode,string name) {

			//データテーブル取得
			var dtaa=this.getData("SELECT * FROM permissions_inheritance WHERE type="+mode.ToString()+" AND child='"+name+"'",this.getInheritanceTN(mode,name),mode,name);
			dtaa.commiting+=this.forInheritance;
			return dtaa;

		}

		/// <summary>
		/// データ取得
		/// </summary>
		/// <param name="sqlQuery">取得用SQLクエリ1</param>
		/// <param name="tableName">テーブル名</param>
		/// <returns></returns>
		private dataTableAndAdapter getData(string sqlQuery,string tableName,int mode,string name) {

			//データアダプタ取得
			var adapter=new MySql.Data.MySqlClient.MySqlDataAdapter(sqlQuery,this.connection);

			//データセット生成
			var dataSet=new System.Data.DataSet();
			adapter.Fill(dataSet);

			dataSet.Tables[0].TableName=tableName;

			return new dataTableAndAdapter(adapter,dataSet.Tables[0],mode,name);
		}

		#endregion

		#region トランザクションコントロール

		/// <summary>
		/// トランザクションコミット
		/// </summary>
		public void commit() {

			//トランザクションをコミット
			this.transaction.Commit();

			//次トランザクションを開始
			this.transaction=this.connection.BeginTransaction();

		}

		/// <summary>
		/// トランザクションロールバック
		/// </summary>
		public void rollback() {

			//トランザクションをロールバック
			this.transaction.Rollback();

			//次トランザクションを開始
			this.transaction=this.connection.BeginTransaction();

		}

		#endregion

		#region コネクションコントロール

		/// <summary>
		/// 接続をオープン
		/// </summary>
		public void open() {

			//接続をオープン
			this.connection.Open();

			//トランザクション開始
			this.transaction=this.connection.BeginTransaction();

		}

		/// <summary>
		/// 接続をクローズ
		/// </summary>
		public void close() {

			//トランザクションをロールバック
			this.rollback();

			//コネクションをクローズ
			this.connection.Close();

		}

		#endregion

		#region 受け渡し用クラス

		public class dataTableAndAdapter {
			public dataTableAndAdapter(MySql.Data.MySqlClient.MySqlDataAdapter adapter,System.Data.DataTable dt,int mode,string name) {
				this.adapter=adapter;
				this.dataTable=dt;
				this.mode=mode;
				this.name=name;
			}

			private int mode {
				get;
				set;
			}

			private string name {
				get;
				set;
			}

			private MySql.Data.MySqlClient.MySqlDataAdapter adapter {
				get;
				set;
			}

			public System.Data.DataTable dataTable {
				get;
				private set;
			}

			public void commit() {
				this.commiting(this.dataTable,new Dictionary<string,string>(){{"mode",this.mode.ToString()},{"name",this.name}});
				var commandBuilder=new MySql.Data.MySqlClient.MySqlCommandBuilder();
				commandBuilder.DataAdapter=this.adapter;
				this.adapter.Update(this.dataTable);
			}

			public event EventHandler<Dictionary<string,string>> commiting;

		}

		#endregion

		#region コミットイベントハンドラ

		private void forPermissions(object sender,Dictionary<string,string> eventArgs) {

			var dt=((System.Data.DataTable)sender);
			foreach (System.Data.DataRow  dr in dt.Rows){
				try {
					dr["name"]=eventArgs["name"];
					dr["type"]=eventArgs["mode"]=="1";
				} catch(Exception ex) {
				}
			}
			
		}

		private void forEntityList(object sender,Dictionary<string,string> eventArgs) {
			var dt=((System.Data.DataTable)sender);
			foreach(System.Data.DataRow dr in dt.Rows) {
				try {
					dr["type"]=eventArgs["mode"]=="1";
				} catch(Exception ex) {
				}
			}
		}

		private void forInheritance(object sender,Dictionary<string,string> eventArgs) {
			var dt=((System.Data.DataTable)sender);
			foreach(System.Data.DataRow dr in dt.Rows) {
				try {
					dr["child"]=eventArgs["name"];
					dr["type"]=eventArgs["mode"]=="1";
				} catch(Exception ex) {
				}
			}
		}

		#endregion

	}
}